#include <linux/types.h>
#include <scsi/scsi_cmnd.h>

#include "debug.h"
#include "scsi_codes.h"
#include "scsidump_ctlr.h"
#include "scsidump.h"
#include "scsi_hooker.h"

extern struct scsi_dmp_ctlr dump_ctlr;


int default_hook_fn(struct scsi_cmnd *cmd,
					void (*done)(struct scsi_cmnd *)){
	char * cmdname;
	if(cmd == NULL){
		return -EINVAL;
	}

	cmdname = scsi_code_to_name(cmd->cmnd[0]);
	printk("[SCSIDUMP] host%d send command %s, code %x\n", cmd->device->host->host_no, cmdname,cmd->cmnd[0]);
	return 0;
}


struct scsi_hooker * alloc_hooker(void){
	struct scsi_hooker *  hooker;

	hooker = (struct scsi_hooker *) kmalloc(sizeof(struct scsi_hooker), GFP_KERNEL);
	if(hooker == NULL){
			printk("memory allocation error\n");
			return NULL;
	}

	hooker->host = NULL;
	hooker->queuecommand_backup = NULL;
	hooker->hooker_fn = default_hook_fn;

	return hooker;
}

void add_hooker(struct scsi_hooker * hooker){

	if(hooker == NULL){
		return ;
	}
	kdebug("hooker->list %p, dump_ctlr %p\n",&hooker->list, &dump_ctlr.scsi_hooker_list);
	write_lock(&dump_ctlr.lock);
	list_add(&hooker->list, &dump_ctlr.scsi_hooker_list);
	write_unlock(&dump_ctlr.lock);

	return;

}

void free_hooker(struct scsi_hooker * hooker){
	if(hooker == NULL){
		return;
	}


	kfree(hooker);
	return;
}


struct scsi_hooker * get_scsihooker_by_hostno(int hostno){
		struct scsi_hooker * hooker = NULL;
		struct list_head * listhead;

		if(hostno < 0 ||hostno >= MAXHOSTS){
			return NULL;
		}

		read_lock(&dump_ctlr.lock);
		if(list_empty(&dump_ctlr.scsi_hooker_list)){
			read_unlock(&dump_ctlr.lock);
			return NULL;
		}

		list_for_each(listhead,&dump_ctlr.scsi_hooker_list){
			hooker = container_of(listhead,scsi_hooker_t,list);
			if(hooker->host->host_no == hostno){
				read_unlock(&dump_ctlr.lock);
				return hooker;
			}
		}

		read_unlock(&dump_ctlr.lock);
		return NULL;


}



