
#include <scsi/scsi_cmnd.h>
#include "debug.h"

#include "scsi_codes.h"
#include "filter.h"
#include "scsidump_ioctl.h"
#include "scsi_hooker.h"
#include "scsidump_ctlr.h"
#include "scsidump.h"


extern struct scsi_dmp_ctlr dump_ctlr;

MODULE_LICENSE("GPL");

int hooker_queue_command(struct scsi_cmnd *cmd,
					void (*done)(struct scsi_cmnd *)){

		struct scsi_hooker * hooker = NULL;
		int filtered;
		hooker = get_scsihooker_by_hostno(cmd->device->host->host_no);

		if(hooker == NULL){
			return DID_NO_CONNECT;
		}
		if(cmd->cmnd[0] == 0x28){
			return 0;

		}
		if(dump_ctlr.filter != NULL && is_filter_enable(dump_ctlr.filter)){
			filtered = do_filter(dump_ctlr.filter, cmd->cmnd[0]);
			if(filtered == FILTER_PASS){//scsi cmd could  pass filter, dump it 
				kdebug("scsi command 0x%x passed\n", cmd->cmnd[0]);
				if(hooker->hooker_fn){
					hooker->hooker_fn(cmd, done);
				}
			}
		}
		
		return hooker->queuecommand_backup(cmd, done);
		

}

static int hook_hosts(void){
	int i = 0;
	struct Scsi_Host * host = NULL;
	struct scsi_hooker * hooker = NULL;

	do{
		host = scsi_host_lookup(i);
		if(IS_ERR(host) || host == NULL){
			break;
		}
		//found host, replace queue_command

		if(host->hostt == NULL || host->hostt->queuecommand == NULL){
			kdebug("found scsi host wthout hostt %p\n", host->hostt);
			continue;
		}
		
		hooker = alloc_hooker();
		if(hooker == NULL){
			return -ENOMEM;
		}
		//replace queue_command
		hooker->queuecommand_backup	= host->hostt->queuecommand;
		hooker->host = host;
		host->hostt->queuecommand = hooker_queue_command;
		kdebug("hook to host%d\n",host->host_no);
		add_hooker(hooker);
		
		i++;
	}while(i < MAXHOSTS);
	return 0;
	
}

void restore_hosts(void){
	struct scsi_hooker * hooker;
	struct list_head * listhead, *tmp;

	write_lock(&dump_ctlr.lock);
	if(list_empty(&dump_ctlr.scsi_hooker_list)){
		write_unlock(&dump_ctlr.lock);
		return;
	}
	list_for_each_safe(listhead, tmp,&dump_ctlr.scsi_hooker_list){
		hooker = container_of(listhead, scsi_hooker_t,list);
		kdebug("del hooker:%p\n", hooker);
		hooker->host->hostt->queuecommand = hooker->queuecommand_backup;
		list_del(&hooker->list);
		free_hooker(hooker);
	}
	write_unlock(&dump_ctlr.lock);
	return;
}

int __init scsi_dump_init(void){
	init_scsi_dmp_ctlr();
	scsidump_interface_init();
	return hook_hosts();
}

void scsi_dump_exit(void){
	restore_hosts();
	scsidump_interface_exit();

	return;
}

module_init(scsi_dump_init);
module_exit(scsi_dump_exit);
