/*
 * =====================================================================================
 *
 *       Filename:  main.c
 *
 *    Description:  

 *
 *        Version:  1.0
 *        Created:  06/07/2009 08:00:27 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Xie Gang
 *        Company: SCU
 *
 * =====================================================================================
 */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>

#include <sys/ioctl.h>


#include "scsi_codes.h"
#include "scsidump_ioctl.h"

#define CTLDEV "/dev/dumpscsi"

#define RULE_EXC 1
#define RULE_INC 2
#define IS_RULE_VALID(rule)  ((rule==RULE_EXC)||(rule == RULE_INC))
#define FILTER_ENABLE 1
#define FILTER_DISABLE 0

int open_ctl_dev(void){

	int fd;

	fd = open(CTLDEV, O_RDWR);
	if(fd < 0){
		printf("can not open ctl dev\n");
		return -1;
	}

	return fd;
}

int filter_enable(int ctlfd){
	struct scsidump_ioctl ioctldata;

	int err;
	ioctldata.type = SCSIDUMP_IOCTL_TYPE_FILTER;

	if(ctlfd < 0){
		printf("wrong ctl fd\n");
		return -1;
	}

	if((err = ioctl(ctlfd, SCSIDUMP_FILTER_ENABLE,&ioctldata)) < 0){
		printf("ioctl error %d\n", errno);
	}
			
	return err;
}

int filter_disable(int ctlfd){
	struct scsidump_ioctl ioctldata;
	int err;
	ioctldata.type = SCSIDUMP_IOCTL_TYPE_FILTER;

	if(ctlfd < 0){
		printf("wrong ctl fd\n");
		return -1;
	}

	if((err = ioctl(ctlfd, SCSIDUMP_FILTER_DISABLE,&ioctldata)) < 0){
		printf("ioctl error %d\n", errno);
	}
			
	return err;

}

int filter_get(int ctlfd){
	struct scsidump_ioctl * ioctldata;
	int err;
	int i;
	

	if(ctlfd < 0){
		printf("wrong ctl fd\n");
		return -1;
	}

	ioctldata = (struct scsidump_ioctl *) malloc (sizeof(struct scsidump_ioctl) + SCSIDUMP_MAX_CODES_LEN + 1);

	if(ioctldata == NULL){
		return -1;
	}
	ioctldata->type = SCSIDUMP_IOCTL_TYPE_FILTER;
	ioctldata->datasize = SCSIDUMP_MAX_CODES_LEN;
	if((err = ioctl(ctlfd, SCSIDUMP_FILTER_GET,ioctldata)) < 0){
		printf("ioctl error %d\n", errno);
		return -1;
	}

	printf("filter rule: %d \n", (int)ioctldata->data[0]);
	if(ioctldata->data[1] == 1){
		printf("state: enable\n");
	}
	else{
		printf("state: disable\n");
	}
	
	printf("codes: ");
	for(i = 2; i < ioctldata->datasize; i ++){
		printf("0x%x  ", ioctldata->data[i]);
	}
	printf("\n");

	free(ioctldata);
	return err;

}

int filter_modify(int ctlfd, int rule, char *codes, int codeslen){
	char * buf, *p;
	int i;
	long code;
	struct scsidump_ioctl  *ioctldata;
	int err;

	//printf("modify filter: rule %x codes %s. codeslen %d\n", rule, codes,codeslen);
	
	
	if(rule != RULE_EXC && rule != RULE_INC ){
		printf("wrong rule %d\n", rule);
		return -1;
	}

	i = 0;
	buf = NULL;
	if(codes != NULL){
		buf = (char *) calloc (codeslen , 1);
		if(buf == NULL){
			return -1;
		}

		p = strtok(codes, ",");
		while(p != NULL){
			code = strtol( p, NULL, 16);
			buf[i] = (char) code;
			i++;
			p = strtok(NULL, ",");

		}
	}
	

	ioctldata = (struct scsidump_ioctl *) calloc (sizeof(struct scsidump_ioctl) + i + 1, 1);
	
	ioctldata->type = SCSIDUMP_IOCTL_TYPE_FILTER;
	ioctldata->datasize = i + 1;
	ioctldata->data[0] = (char)rule;
	//codes could be null
	if( i != 0 ){
		memcpy(ioctldata->data + 1, buf, i );
	}

	if((err = ioctl(ctlfd, SCSIDUMP_FILTER_MODIFY, ioctldata)) < 0){
		printf("ioctl error %d\n", errno);
	}


	free(buf);
	free(ioctldata);
	return err;


}

void print_usage(void){
	printf("usage:\n");
	printf("-[edmrcgh]:\n");
	printf("-e: enable filter. Defaut is filter all\n");
	printf("-d: disable filter\n");
	printf("-m: modifiy filter. must followed by -m or -r\n");
	printf("-r: specify new rule. 1 exclude, 2 include\n");
	printf("-c: scsi op code, seprated with ','. For example, 0x22,0x21\n");
	printf("-g: display current filter\n");
	printf("-h: print this list\n");
}

/*
* 
   -e :enable
   -d :disable
   -m -r rule -c codes
   -g :get filter
*/

int main(int argc, char * argv[]){
	int fd;
	int oc, mflag, rule, codeslen;
	char * codes;

	mflag = 0;
	rule = RULE_EXC;
	codes = NULL;
	codeslen = 0;

	
	fd = open_ctl_dev();
		if(fd < 0){
			return -1;
		}

	//parse parameters
	while( (oc = getopt(argc, argv, "edgmr:c:")) != -1){
		switch(oc){
			case 'e':
				return 	filter_enable(fd);
			case 'd':
				return	 filter_disable(fd);
			case 'g':
				return filter_get(fd);
			case 'm':
				mflag = 1;
				break;
			case 'r':
				rule = atoi(optarg);
				break;
			case 'c':
				codes = optarg;
				break;
			default:
				print_usage();
				return -1;
		}
	}
	
	
	//modify filter
	if(mflag != 0 ){
		if(!IS_RULE_VALID(rule)){
			printf("Invalid rule \n");
			print_usage();
			return -1;
		}
		if(codes != NULL){
			codeslen = strlen(codes);
		}
		
		return filter_modify(fd,rule, codes, codeslen);
	}

	print_usage();
	return -1;
						
}

